@@ -28,6 +28,9 @@ class RequestType:
 
 class RedactMode:
     """RedactMode Options"""
+    ALLOWLIST = 'allow'
+    BLOCKLIST = 'block'
 
+    # DEPRECATED NAMES
     BLACKLIST = 'black'
     WHITELIST = 'white'
