@@ -8,32 +8,36 @@ functions.
 """
 
 # Exceptions
-from .Exceptions import FailedToSendResponseException
-from .Exceptions import DataIsNotDictException
-from .Exceptions import InvalidResponseStatusException
-from .Exceptions import NotValidRequestObjectException
+from accustom.Exceptions import FailedToSendResponseException
+from accustom.Exceptions import DataIsNotDictException
+from accustom.Exceptions import InvalidResponseStatusException
+from accustom.Exceptions import NotValidRequestObjectException
+from accustom.Exceptions import ResponseTooLongException
+from accustom.Exceptions import NoPhysicalResourceIdException
 
 # Constants
-from .constants import RequestType
-from .constants import Status
+from accustom.constants import RequestType
+from accustom.constants import Status
 
 # Response
-from .response import is_valid_event
+from accustom.response import is_valid_event
 
 # RedactionConfig
-from .redaction import RedactionConfig
-from .redaction import StandaloneRedactionConfig
+from accustom.redaction import RedactionConfig
+from accustom.redaction import StandaloneRedactionConfig
 
 # Required Libraries
 import logging
 import json
 from functools import wraps
 from uuid import uuid4
-from .response import ResponseObject
+from accustom.response import ResponseObject
 import six
 from boto3 import client
 from botocore.client import Config
-from botocore import exceptions as bexceptions
+from botocore import exceptions as boto_exceptions
+from typing import Callable, TypeVar, Any, Union
+from typing_extensions import ParamSpec
 
 logger = logging.getLogger(__name__)
 
@@ -44,6 +48,8 @@ except ImportError:
     from botocore.vendored import requests
     logger.warning("botocore.vendored version of requests is deprecated. Please include requests in your code bundle.")
 
+_T = TypeVar('_T')
+_P = ParamSpec('_P')
 
 # Time in milliseconds to set the alarm for (in milliseconds)
 # Should be set to twice the worst case response time to send to S3
@@ -52,7 +58,7 @@ TIMEOUT_THRESHOLD = 2000
 
 
 def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool = False,
-              redactConfig: RedactionConfig = None, timeoutFunction: bool = False):
+              redactConfig: RedactionConfig = None, timeoutFunction: bool = False) -> Callable[_P, _T]:
     """Decorate a function to add exception handling and emit CloudFormation responses.
 
     Usage with Lambda:
@@ -94,9 +100,9 @@ def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool =
 
     """
 
-    def inner_decorator(func):
+    def inner_decorator(func: Callable[_P, _T]) -> Callable[_P, _T]:
         @wraps(func)
-        def handler_wrapper(event: dict, context: dict = None):
+        def handler_wrapper(event: dict, context: Any, *args, **kwargs) -> Union[dict, str]:
             nonlocal redactConfig
             nonlocal timeoutFunction
             logger.info('Request received, processing...')
@@ -109,69 +115,85 @@ def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool =
 
             # Timeout Function Handler
             if 'LambdaParentRequestId' in event:
-                logger.info('This request has been invoked as a child, for parent logs please see request ID: %s' %
-                            event['LambdaParentRequestId'])
+                logger.info(
+                    f'This request has been invoked as a child, for parent logs please see request ID: '
+                    f'{event["LambdaParentRequestId"]}'
+                    )
             elif context is None and timeoutFunction:
-                logger.warning('You cannot use the timeoutFunction option outside of Lambda. To suppress this warning' +
-                               ', set timeoutFunction to False')
+                logger.warning(
+                    'You cannot use the timeoutFunction option outside of Lambda. To suppress this warning' +
+                    ', set timeoutFunction to False'
+                    )
             elif timeoutFunction:
                 # Attempt to invoke the function. Depending on the error we get may continue execution or return
                 logger.info('Request has been invoked in Lambda with timeoutFunction set, attempting to invoke self')
-                pevent = event.copy()
-                pevent['LambdaParentRequestId'] = context.aws_request_id
-                payload = json.dumps(pevent).encode('UTF-8')
+                p_event = event.copy()
+                p_event['LambdaParentRequestId'] = context.aws_request_id
+                payload = json.dumps(p_event).encode('UTF-8')
                 timeout = (context.get_remaining_time_in_millis() - TIMEOUT_THRESHOLD) / 1000
-                # Edge case where time is set to very low timeout, use half the timeout threshold as the timeout for the
+                # Edge case where time is set to very low timeout, use half the timeout threshold as the timeout for
                 # the Lambda Function
                 if timeout <= 0: timeout = TIMEOUT_THRESHOLD / 2000
                 config = Config(connect_timeout=2, read_timeout=timeout, retries={'max_attempts': 0})
                 b_lambda = client('lambda', config=config)
 
-                # Normally we would just do a catch all error handler but in this case we want to be paranoid
+                # Normally we would just do a catch-all error handler but in this case we want to be paranoid
                 try:
-                    response = b_lambda.invoke(FunctionName=context.invoked_function_arn,
-                                               InvocationType='RequestResponse', Payload=payload)
+                    response = b_lambda.invoke(
+                        FunctionName=context.invoked_function_arn,
+                        InvocationType='RequestResponse', Payload=payload
+                        )
                     # Further checks
                     if 'FunctionError' in response:
                         response.get('Payload', ''.encode('UTF-8'))
-                        message = 'Invocation got an error: %s' % payload.decode()
+                        message = f'Invocation got an error: {payload.decode()}'
                         logger.error(message)
                         return ResponseObject(reason=message, responseStatus=Status.FAILED).send(event, context)
                     else:
                         # In this case the function returned without error which means we can assume the chained
-                        # invokation sent a response, so we do not have too.
-                        logger.info('Compeleted execution of chained invocation, returning payload')
+                        # invocation sent a response, so we do not have too.
+                        logger.info('Completed execution of chained invocation, returning payload')
                         response.get('Payload', ''.encode('UTF-8'))
                         return payload.decode()
 
-                except bexceptions.ClientError as e:
-                    logger.warning('Caught exception %s while trying to invoke function. Running handler locally.'
-                                   % str(e))
-                    logger.warning('You cannot use the timeoutFunction option without the ability for the function to' +
-                                   ' invoke itself. To suppress this warning, set timeoutFunction to False')
-                except bexceptions.ConnectionError as e:
-                    logger.error('Got error %s while trying to invoke function. Running handler locally' % str(e))
-                    logger.error('You cannot use the timeoutFunction option without the ability to connect to the ' +
-                                 'Lambda API from within the function. As we may not have time to execute the ' +
-                                 'function, returning failure.')
-                    return ResponseObject(reason='Unable to call Lambda to do chained invoke, returning failure.',
-                                          responseStatus=Status.FAILED).send(event, context)
-                except bexceptions.ReadTimeoutError:
+                except boto_exceptions.ClientError as e:
+                    logger.warning(
+                        f'Caught exception {str(e)} while trying to invoke function. Running handler locally.'
+                        )
+                    logger.warning(
+                        'You cannot use the timeoutFunction option without the ability for the function to' +
+                        ' invoke itself. To suppress this warning, set timeoutFunction to False'
+                        )
+                except boto_exceptions.ConnectionError as e:
+                    logger.error(f'Got error {str(e)} while trying to invoke function. Running handler locally')
+                    logger.error(
+                        'You cannot use the timeoutFunction option without the ability to connect to the ' +
+                        'Lambda API from within the function. As we may not have time to execute the ' +
+                        'function, returning failure.'
+                        )
+                    return ResponseObject(
+                        reason='Unable to call Lambda to do chained invoke, returning failure.',
+                        responseStatus=Status.FAILED
+                        ).send(event, context)
+                except boto_exceptions.ReadTimeoutError:
                     # This should be a critical failure
                     logger.error('Waited the read timeout and function did not return, returning an error')
-                    return ResponseObject(reason='Lambda function timed out, returning failure.',
-                                          responseStatus=Status.FAILED).send(event, context)
+                    return ResponseObject(
+                        reason='Lambda function timed out, returning failure.',
+                        responseStatus=Status.FAILED
+                        ).send(event, context)
                 except Exception as e:
-                    message = 'Got an %s I did not understand while trying to invoke child function: %s' % (e.__class__,
-                                                                                                            str(e))
+                    message = f'Got an {e.__class__} I did not understand while trying to invoke child function: ' \
+                              f'{str(e)}'
                     logger.error(message)
                     return ResponseObject(reason=message, responseStatus=Status.FAILED).send(event, context)
 
             # Debug Logging Handler
             if logger.getEffectiveLevel() <= logging.DEBUG:
                 if context is not None:
-                    logger.debug('Running request with Lambda RequestId: %s' % context.aws_request_id)
+                    logger.debug(f'Running request with Lambda RequestId: {context.aws_request_id}')
                 if redactConfig is not None and isinstance(redactConfig, (StandaloneRedactionConfig, RedactionConfig)):
+                    # noinspection PyProtectedMember
                     logger.debug('Request Body:\n' + json.dumps(redactConfig._redact(event)))
                 elif redactConfig is not None:
                     logger.warning('A non valid RedactionConfig was provided, and ignored')
@@ -180,19 +202,17 @@ def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool =
                     logger.debug('Request Body:\n' + json.dumps(event))
 
             try:
-                logger.info('Running CloudFormation request %s for stack: %s' % (event['RequestId'], event['StackId']))
+                logger.info(f'Running CloudFormation request {event["RequestId"]} for stack: {event["StackId"]}')
                 # Run the function
-                if context is not None:
-                    result = func(event, context)
-                else:
-                    result = func(event)
+                result = func(event, context, *args, **kwargs)
 
             except Exception as e:
                 # If there was an exception thrown by the function, send a failure response
                 result = ResponseObject(
                     physicalResourceId=str(uuid4()) if context is None else None,
-                    reason='Function %s failed due to exception "%s"' % (func.__name__, str(e)),
-                    responseStatus=Status.FAILED)
+                    reason=f'Function {func.__name__} failed due to exception "{str(e)}"',
+                    responseStatus=Status.FAILED
+                    )
                 logger.error(result.reason)
 
             if not isinstance(result, ResponseObject):
@@ -200,20 +220,23 @@ def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool =
                 # failure if it is an invalid response type, or if the enforceUseOfClass is explicitly or implicitly set
                 if context is None:
                     result = ResponseObject(
-                        reason='Response Object of type %s was not a ResponseObject and there is no Lambda Context'
-                               % result.__class__,
-                        responseStatus=Status.FAILED)
+                        reason=f'Response Object of type {result.__class__} was not a ResponseObject and there is no '
+                               f'Lambda Context',
+                        responseStatus=Status.FAILED
+                        )
                     logger.error(result.reason)
                 elif enforceUseOfClass:
                     result = ResponseObject(
-                        reason='Response Object of type %s was not a ResponseObject instance and ' +
-                               'enforceUseOfClass set to true' % result.__class__,
-                        responseStatus=Status.FAILED)
+                        reason=f'Response Object of type {result.__class__} was not a ResponseObject instance and '
+                               f'enforceUseOfClass set to true',
+                        responseStatus=Status.FAILED
+                        )
                     logger.error(result.reason)
                 elif result is False:
                     result = ResponseObject(
-                        reason='Function %s returned False.' % func.__name__,
-                        responseStatus=Status.FAILED)
+                        reason=f'Function {func.__name__} returned False.',
+                        responseStatus=Status.FAILED
+                        )
                     logger.debug(result.reason)
                 elif isinstance(result, dict):
                     result = ResponseObject(data=result)
@@ -223,9 +246,9 @@ def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool =
                     result = ResponseObject()
                 else:
                     result = ResponseObject(
-                        reason='Return value from Function %s is of unsupported type %s' % (func.__name__,
-                                                                                            result.__class__),
-                        responseStatus=Status.FAILED)
+                        reason=f'Return value from Function {func.__name__} is of unsupported type {result.__class__}',
+                        responseStatus=Status.FAILED
+                        )
                     logger.error(result.reason)
 
             # This block will hide resources on delete failure if the flag is set to true
@@ -237,35 +260,67 @@ def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool =
                         logger.debug('Data:\n' + json.dumps(result.data))
                     else:
                         logger.debug('Data: [REDACTED]')
-                if result.reason is not None: logger.debug('Reason: %s' % result.reason)
-                if result.physicalResourceId is not None: logger.debug('PhysicalResourceId: %s'
-                                                                       % result.physicalResourceId)
+                if result.reason is not None: logger.debug(f'Reason: {result.reason}')
+                if result.physicalResourceId is not None: logger.debug(
+                    f'PhysicalResourceId: {result.physicalResourceId}'
+                    )
                 result = ResponseObject(
                     reason='There may be resources created by this Custom Resource that have not been cleaned' +
                            'up despite the fact this resource is in DELETE_COMPLETE',
                     physicalResourceId=result.physicalResourceId,
-                    responseStatus=Status.SUCCESS)
+                    responseStatus=Status.SUCCESS
+                    )
 
             try:
                 return_value = result.send(event, context)
-            except Exception as e:
-                if isinstance(e, FailedToSendResponseException):
-                    raise e
-                logger.error('Malformed request, Exception: %s' % str(e))
-                if result.data is not None and not isinstance(e, DataIsNotDictException):
-                    if not result.squashPrintResponse:
-                        logger.debug('Data:\n' + json.dumps(result.data))
-                    else:
-                        logger.debug('Data: [REDACTED]')
-                if result.reason is not None: logger.debug('Reason: %s' % result.reason)
-                if result.physicalResourceId is not None: logger.debug('PhysicalResourceId: %s'
-                                                                       % result.physicalResourceId)
-                if not isinstance(e, InvalidResponseStatusException): logger.debug('Status: %s'
-                                                                                   % result.responseStatus)
+            except NoPhysicalResourceIdException:
+                message = "An unexpected error has occurred, No Physical Resource ID provided in response."
+                logger.error(message)
                 result = ResponseObject(
-                    reason='Malformed request, Exception: %s' % str(e),
+                    reason=message,
                     physicalResourceId=result.physicalResourceId,
-                    responseStatus=Status.FAILED)
+                    responseStatus=Status.FAILED
+                    )
+                return_value = result.send(event, context)
+            except InvalidResponseStatusException:
+                message = f'Status provided "{result.responseStatus}" is not a valid status.'
+                logger.error(message)
+                result = ResponseObject(
+                    reason=message,
+                    physicalResourceId=result.physicalResourceId,
+                    responseStatus=Status.FAILED
+                    )
+                return_value = result.send(event, context)
+            except DataIsNotDictException as e:
+                message = f'Malformed Data Block in Response, Exception; {str(e)}'
+                logger.error(message)
+                result = ResponseObject(
+                    reason=message,
+                    physicalResourceId=result.physicalResourceId,
+                    responseStatus=Status.FAILED
+                    )
+                return_value = result.send(event, context)
+            except ResponseTooLongException as e:
+                message = str(e)
+                logger.error(message)
+                result = ResponseObject(
+                    reason=message,
+                    physicalResourceId=result.physicalResourceId,
+                    responseStatus=Status.FAILED
+                    )
+                return_value = result.send(event, context)
+            except FailedToSendResponseException as e:
+                # Capturing and re-raising exception to prevent generic exception handler from kicking in
+                raise e
+            except Exception as e:
+                # Generic error capture
+                message = f'Malformed request, Exception: {str(e)}'
+                logger.error(message)
+                result = ResponseObject(
+                    reason=message,
+                    physicalResourceId=result.physicalResourceId,
+                    responseStatus=Status.FAILED
+                    )
                 return_value = result.send(event, context)
             return return_value
 
@@ -274,7 +329,9 @@ def decorator(enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool =
     return inner_decorator
 
 
-def rdecorator(decoratorHandleDelete: bool = False, expectedProperties: list = None, genUUID: bool = True):
+def rdecorator(decoratorHandleDelete: bool = False,
+               expectedProperties: list = None,
+               genUUID: bool = True) -> Callable[_P, _T]:
     """Decorate a function to add input validation for resource handler functions.
 
         Usage with Lambda:
@@ -320,35 +377,37 @@ def rdecorator(decoratorHandleDelete: bool = False, expectedProperties: list = N
             event (dict): The request object being processed (Required).
     """
 
-    def resource_decorator_inner(func):
+    def resource_decorator_inner(func: Callable[_P, _T]) -> Callable[_P, _T]:
         @wraps(func)
-        def resource_decorator_handler(event: dict, *args, **kwargs):
+        def resource_decorator_handler(event: dict, *args, **kwargs) -> Union[ResponseObject, dict, str]:
             if not is_valid_event(event):
                 # If it is not a valid event we need to raise an exception
                 message = 'The event object passed is not a valid Request Object as per ' + \
                           'https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/crpg-ref-requests.html'
                 logger.error(message)
                 raise NotValidRequestObjectException(message)
-            logger.info('Supported resource %s' % event['ResourceType'])
+            logger.info(f'Supported resource {event["ResourceType"]}')
 
             # Set the Physical Resource ID to a randomly generated UUID if it is not present
             if genUUID and 'PhysicalResourceId' not in event:
                 event['PhysicalResourceId'] = str(uuid4())
-                logger.info('Set PhysicalResourceId to %s' % event['PhysicalResourceId'])
+                logger.info(f'Set PhysicalResourceId to {event["PhysicalResourceId"]}')
 
             # Handle Delete when decoratorHandleDelete is set to True
             if decoratorHandleDelete and event['RequestType'] == RequestType.DELETE:
-                logger.info('Request type %s detected, returning success without calling function' % RequestType.DELETE)
+                logger.info(f'Request type {RequestType.DELETE} detected, returning success without calling function')
                 return ResponseObject(physicalResourceId=event['PhysicalResourceId'])
 
             # Validate important properties exist
             if expectedProperties is not None and isinstance(expectedProperties, (list, tuple)):
                 for index, item in enumerate(expectedProperties):
                     if item not in event['ResourceProperties']:
-                        err_msg = 'Property %s missing, sending failure signal' % item
+                        err_msg = f'Property {item} missing, sending failure signal'
                         logger.info(err_msg)
-                        return ResponseObject(reason=err_msg, responseStatus=Status.FAILED,
-                                              physicalResourceId=event['PhysicalResourceId'])
+                        return ResponseObject(
+                            reason=err_msg, responseStatus=Status.FAILED,
+                            physicalResourceId=event['PhysicalResourceId']
+                            )
 
             # If a list or tuple was not provided then log a warning
             elif expectedProperties is not None:
@@ -364,7 +423,7 @@ def rdecorator(decoratorHandleDelete: bool = False, expectedProperties: list = N
 
 def sdecorator(decoratorHandleDelete: bool = False, expectedProperties: list = None, genUUID: bool = True,
                enforceUseOfClass: bool = False, hideResourceDeleteFailure: bool = False,
-               redactConfig: RedactionConfig = None, timeoutFunction: bool = True):
+               redactConfig: RedactionConfig = None, timeoutFunction: bool = True) -> Callable[_P, _T]:
     """Decorate a function to add input validation for resource handler functions, exception handling and send
     CloudFormation responses.
 
@@ -408,17 +467,20 @@ def sdecorator(decoratorHandleDelete: bool = False, expectedProperties: list = N
          FailedToSendResponseException
          NotValidRequestObjectException
     """
-    if not isinstance(redactConfig, StandaloneRedactionConfig) and logger.getEffectiveLevel() <= logging.DEBUG:
+    if redactConfig is not None and not isinstance(redactConfig, StandaloneRedactionConfig) and \
+            logger.getEffectiveLevel() <= logging.DEBUG:
         logger.warning('A non valid StandaloneRedactionConfig was provided, and ignored')
         redactConfig = None
 
-    def standalone_decorator_inner(func):
+    def standalone_decorator_inner(func: Callable[_P, _T]) -> Callable[_P, _T]:
         @wraps(func)
-        @decorator(enforceUseOfClass=enforceUseOfClass, hideResourceDeleteFailure=hideResourceDeleteFailure,
-                   redactConfig=redactConfig, timeoutFunction=timeoutFunction)
+        @decorator(
+            enforceUseOfClass=enforceUseOfClass, hideResourceDeleteFailure=hideResourceDeleteFailure,
+            redactConfig=redactConfig, timeoutFunction=timeoutFunction
+            )
         @rdecorator(decoratorHandleDelete=decoratorHandleDelete, expectedProperties=expectedProperties, genUUID=genUUID)
-        def standalone_decorator_handler(event: dict, context: dict = None):
-            return func(event, context)
+        def standalone_decorator_handler(event: dict, context: Any, *args, **kwargs) -> Union[dict, str]:
+            return func(event, context, *args, **kwargs)
 
         return standalone_decorator_handler
 
